﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Linq;
using Tanshu.Accounts.Contracts;
using Tanshu.Data;

namespace Tanshu.Accounts.Helpers
{
    public class SelectProduct : Tanshu.Data.BaseSelector<ProductDisplaySmallBO>
    {
        public event ProductEventHandler productEvent;
        public SelectProduct(GetData<ProductDisplaySmallBO> getData, bool autoClose)
            : base(getData, autoClose, "List of Products")
        {
            List<string> filters = new List<string>();
            filters.Add("Name");
            filters.Add("ProductGroup");
            SetFilterColumns(filters);
            grid.Columns["ProductID"].Visible = false;
        }

        protected override void FilterChanged(Dictionary<string, string> filter)
        {
            data = getData(filter);
            bindingSource.DataSource = data;
        }

        protected override void UpdateDisplay(ProductDisplaySmallBO item)
        {
            if (item == null)
                DisplayLabel = "";
            else
                DisplayLabel = string.Format("Chosen Product {0} with rate Rs. {1}", item.Name, item.Price);
        }
        protected override ProductDisplaySmallBO HandleKeydown(object sender, ExtendedKeyEventArgs e)
        {
            var product = bindingSource.Current as ProductDisplaySmallBO;
            if (productEvent == null)
            {
                e.Handled = false;
                return null;
            }
            Guid? id = null;
            if ((product != null) && (e.KeyCode == Keys.F2))
                id = product.ProductID;

            if ((e.KeyCode == Keys.F1) || (e.KeyCode == Keys.F2))
            {
                var updatedProduct = productEvent(sender, new ProductEventArgs(id));
                if (updatedProduct != null)
                product = new ProductDisplaySmallBO() {  ProductID = updatedProduct.ProductID, Code = product.Code };
                e.Handled = product != null;
            }
            return product;
        }
        #region Designer Code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
        }

        #endregion
        #endregion
    }
    public delegate ProductBO ProductEventHandler(object sender, ProductEventArgs e);
    public class ProductEventArgs : EventArgs
    {
        public ProductEventArgs(Guid? productID)
        {
            ProductID = productID;
        }
        public Guid? ProductID
        {
            get;
            private set;
        }
    }
}
