﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Linq;
using Tanshu.Accounts.Contracts;
using Tanshu.Data;

namespace Tanshu.Accounts.Helpers
{
    public class SelectUser : Tanshu.Data.BaseSelector<UserBO>
    {
        public event UserEventHandler userEvent;
        public SelectUser(GetData<UserBO> getData, bool autoClose) : base (getData, true, "Select a User") 
        {
            List<string> filters = new List<string>();
            filters.Add("Name");
            SetFilterColumns(filters);
            grid.Columns["UserID"].Visible = false;
        }

        protected override void FilterChanged(Dictionary<string, string> filter)
        {
            //data = originalData.Where(d => d.Name.ToLower().Contains(filter["Name"].ToLower())).ToList();
            data = getData(filter);
            bindingSource.DataSource = data;
        }
        protected override void UpdateDisplay(UserBO item)
        {
            if (item == null)
                DisplayLabel = "";
            else
                DisplayLabel = string.Format("User Name: {0}", item.Name);
        }
        protected override UserBO HandleKeydown(object sender, ExtendedKeyEventArgs e)
        {
            UserBO user = bindingSource.Current as UserBO;

            if (userEvent == null)
            {
                e.Handled = false;
                return user;
            }
            if ((e.KeyCode != Keys.F1) && (e.KeyCode != Keys.F2))
            {
                e.Handled = false;
                return user;
            }
            if ((user == null) && (e.KeyCode == Keys.F2))
            {
                e.Handled = false;
                return user;
            }

            if (e.KeyCode == Keys.F1)
                user = new UserBO();
            UserEventArgs userEventArgs = new UserEventArgs(user);
            user = userEvent(sender, userEventArgs);

            e.Handled = userEventArgs.Handled;
            return user;
        }
        #region Designer Code
        /// <summary>
        /// Required designer variable.
        /// </summary>
        private System.ComponentModel.IContainer components = null;

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            if (disposing && (components != null))
            {
                components.Dispose();
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.components = new System.ComponentModel.Container();
        }

        #endregion
        #endregion
    }
    public delegate UserBO UserEventHandler(object sender, UserEventArgs e);
    public class UserEventArgs : EventArgs
    {
        public UserEventArgs(UserBO user)
        {
            User = user;
            Handled = false;
        }

        public UserBO User
        {
            get;
            private set;
        }

        public bool Handled
        {
            get;
            set;
        }
    }
}
